###############   PSQF 6249 FA & SEM Example 7b CFA Invariance Invariance using R   #############

# Set width of output and  number of digits to be printed
options("width"=140, "digits"=6) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("ggplot2")) install.packages("ggplot2")
library(ggplot2) # To make pictures

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit SEMs

# Other packages not used here, but which are pry useful

#if (!require("semTools")) install.packages("semTools")
#library(semTools) # lavaan add-ons (multiple imputation, moderation)

#if (!require("MplusAutomation")) install.packages("MplusAutomation")
#library(lavaan) # To use Mplus within R

#########################################################################################################

# Define variables for working directory and data name
filesave="C:\\Dropbox/22_PSQF6249/NOSIE/PSQF6249_Example7b/"
filename="Example7b.xlsx"
setwd(dir=filesave)

# Import social functioning indicator-level data
Example7b = read_excel(paste0(filesave,filename)) 
# Convert to data frame as data frame to use for analysis
Example7b = as.data.frame(Example7b)
# Recode negatively worded items
Example7b$v4T1=Example7b$v4T1*(-1); Example7b$v4T2=Example7b$v4T2*(-1); Example7b$v4T3=Example7b$v4T3*(-1)
Example7b$v5T1=Example7b$v5T1*(-1); Example7b$v5T2=Example7b$v5T2*(-1); Example7b$v5T3=Example7b$v5T3*(-1)
Example7b$v6T1=Example7b$v6T1*(-1); Example7b$v6T2=Example7b$v6T2*(-1); Example7b$v6T3=Example7b$v6T3*(-1)

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example7b_CFA_Longitudinal_Invariance_Output.txt"))

#### General notes about lavaan syntax objects:
# Once factors are defined they can be used in syntax as if they were observed variables
# Multiple commands can go on a single line separated by ; 
# Parameters can be fixed to constant values by using the * command
# Start values can be specified by using value* in front of parameter 
# Labels can be implemented with (label)* in front of parameter

# Example of how to concatenate syntax strings into one model
#Model <- sem(model=c(syntax1, syntax2), data=myData)


#################################################################################################################
# Measurement Invariance Model 1: Configural Invariance Model
# all parameters are estimated separately across occasions except those for identification

ConfiguralSyntax="
#================================================================================================================
# Factor loadings all freely estimated at each occasion with labels
Time1 =~ L1T1*v1T1 + L2T1*v2T1 + L3T1*v3T1 + L4T1*v4T1 + L5T1*v5T1 + L6T1*v6T1
Time2 =~ L1T2*v1T2 + L2T2*v2T2 + L3T2*v3T2 + L4T2*v4T2 + L5T2*v5T2 + L6T2*v6T2
Time3 =~ L1T3*v1T3 + L2T3*v2T3 + L3T3*v3T3 + L4T3*v4T3 + L5T3*v5T3 + L6T3*v6T3
#================================================================================================================
# Item intercepts all freely estimated at each occasion with labels
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3T1*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6T1*1 
v1T2 ~ I1T2*1; v2T2 ~ I2T2*1;  v3T2 ~ I3T2*1; v4T2 ~ I4T2*1; v5T2 ~ I5T2*1; v6T2 ~ I6T2*1 
v1T3 ~ I1T3*1; v2T3 ~ I2T3*1;  v3T3 ~ I3T3*1; v4T3 ~ I4T3*1; v5T3 ~ I5T3*1; v6T3 ~ I6T3*1 
#================================================================================================================
# Residual variances all freely estimated at each occasion with labels
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3T1*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1T2*v1T2; v2T2 ~~ E2T2*v2T2;  v3T2 ~~ E3T2*v3T2; v4T2 ~~ E4T2*v4T2; v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6T2*v6T2
v1T3 ~~ E1T3*v1T3; v2T3 ~~ E2T3*v2T3;  v3T3 ~~ E3T3*v3T3; v4T3 ~~ E4T3*v4T3; v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6T3*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at each occasion
Time1 ~~ 1*Time1; Time2 ~~ 1*Time2; Time3 ~~ 1*Time3
#================================================================================================================
# Factor mean fixed to zero for identification at each occasion
Time1 ~ 0; Time2 ~ 0; Time3 ~ 0
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
Configural=lavaan(model=ConfiguralSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(Configural, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 2a: Metric Invariance Model
# constrain factor loadings equal over time, estimate factor variance at T2 and T3

MetricASyntax="
#================================================================================================================
# Factor loadings NOW CONSTRAINED EQUAL OVER TIME VIA SAME LABELS
Time1 =~ L1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2 + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3 + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts all freely estimated at each occasion with labels
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3T1*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6T1*1 
v1T2 ~ I1T2*1; v2T2 ~ I2T2*1;  v3T2 ~ I3T2*1; v4T2 ~ I4T2*1; v5T2 ~ I5T2*1; v6T2 ~ I6T2*1 
v1T3 ~ I1T3*1; v2T3 ~ I2T3*1;  v3T3 ~ I3T3*1; v4T3 ~ I4T3*1; v5T3 ~ I5T3*1; v6T3 ~ I6T3*1 
#================================================================================================================
# Residual variances all freely estimated at each occasion with labels
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3T1*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1T2*v1T2; v2T2 ~~ E2T2*v2T2;  v3T2 ~~ E3T2*v3T2; v4T2 ~~ E4T2*v4T2; v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6T2*v6T2
v1T3 ~~ E1T3*v1T3; v2T3 ~~ E2T3*v2T3;  v3T3 ~~ E3T3*v3T3; v4T3 ~~ E4T3*v4T3; v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6T3*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 ONLY, NOW ESTIMATED AT T2 AND T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at each occasion
Time1 ~ 0; Time2 ~ 0; Time3 ~ 0
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
MetricA=lavaan(model=MetricASyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(MetricA, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual loadings
modificationindices(object=MetricA, op="=~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is MetricA worse than Configural?")
anova(Configural, MetricA)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 2b: Patrial Metric Invariance Model (v1T1 loading free)
# constrain factor loadings equal over time, estimate factor variance at T2 and T3

MetricBSyntax="
#================================================================================================================
# Factor loadings NOW CONSTRAINED EQUAL OVER TIME VIA SAME LABELS EXCEPT v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts all freely estimated at each occasion with labels
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3T1*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6T1*1 
v1T2 ~ I1T2*1; v2T2 ~ I2T2*1;  v3T2 ~ I3T2*1; v4T2 ~ I4T2*1; v5T2 ~ I5T2*1; v6T2 ~ I6T2*1 
v1T3 ~ I1T3*1; v2T3 ~ I2T3*1;  v3T3 ~ I3T3*1; v4T3 ~ I4T3*1; v5T3 ~ I5T3*1; v6T3 ~ I6T3*1 
#================================================================================================================
# Residual variances all freely estimated at each occasion with labels
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3T1*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1T2*v1T2; v2T2 ~~ E2T2*v2T2;  v3T2 ~~ E3T2*v3T2; v4T2 ~~ E4T2*v4T2; v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6T2*v6T2
v1T3 ~~ E1T3*v1T3; v2T3 ~~ E2T3*v2T3;  v3T3 ~~ E3T3*v3T3; v4T3 ~~ E4T3*v4T3; v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6T3*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 ONLY, NOW ESTIMATED AT T2 AND T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at each occasion
Time1 ~ 0; Time2 ~ 0; Time3 ~ 0
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
MetricB=lavaan(model=MetricBSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(MetricB, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual loadings
modificationindices(object=MetricB, op="=~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is MetricB better than MetricA?")
anova(MetricB, MetricA)

print("LRT: Is MetricB worse than Configural?")
anova(Configural, MetricB)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 3a: Scalar Invariance Model
# constrain intercepts equal over time, estimate factor mean at T2 and T3

ScalarASyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts NOW CONSTRAINED EQUAL OVER TIME VIA SAME LABELS EXCEPT v1T1
v1T1 ~ I1T1*1; v2T1 ~ I2*1;  v3T1 ~ I3*1; v4T1 ~ I4*1; v5T1 ~ I5*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;  v3T2 ~ I3*1; v4T2 ~ I4*1; v5T2 ~ I5*1; v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;  v3T3 ~ I3*1; v4T3 ~ I4*1; v5T3 ~ I5*1; v6T3 ~ I6*1 
#================================================================================================================
# Residual variances all freely estimated at each occasion with labels
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3T1*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1T2*v1T2; v2T2 ~~ E2T2*v2T2;  v3T2 ~~ E3T2*v3T2; v4T2 ~~ E4T2*v4T2; v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6T2*v6T2
v1T3 ~~ E1T3*v1T3; v2T3 ~~ E2T3*v2T3;  v3T3 ~~ E3T3*v3T3; v4T3 ~~ E4T3*v4T3; v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6T3*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 only, still estimated at T2 and T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 ONLY, NOW ESTIMATED AT T2 AND T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
ScalarA=lavaan(model=ScalarASyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(ScalarA, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ScalarA, op="~1", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarA worse than MetricB?")
anova(MetricB, ScalarA)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 3b: Scalar Invariance Model (no v1T1 v5T1)
# constrain intercepts equal over time, estimate factor mean at T2 and T3

ScalarBSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts NOW CONSTRAINED EQUAL OVER TIME VIA SAME LABELS EXCEPT v1T1 v5T1
v1T1 ~ I1T1*1; v2T1 ~ I2*1;  v3T1 ~ I3*1; v4T1 ~ I4*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;  v3T2 ~ I3*1; v4T2 ~ I4*1; v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;  v3T3 ~ I3*1; v4T3 ~ I4*1; v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances all freely estimated at each occasion with labels
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3T1*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1T2*v1T2; v2T2 ~~ E2T2*v2T2;  v3T2 ~~ E3T2*v3T2; v4T2 ~~ E4T2*v4T2; v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6T2*v6T2
v1T3 ~~ E1T3*v1T3; v2T3 ~~ E2T3*v2T3;  v3T3 ~~ E3T3*v3T3; v4T3 ~~ E4T3*v4T3; v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6T3*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 only, still estimated at T2 and T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 ONLY, NOW ESTIMATED AT T2 AND T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
ScalarB=lavaan(model=ScalarBSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(ScalarB, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ScalarB, op="~1", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarB better than ScalarA?")
anova(ScalarB, ScalarA)

print("LRT: Is ScalarB worse than MetricB?")
anova(MetricB, ScalarB)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 3c: Scalar Invariance Model (no v1T1 v5T1 v4T1)
# constrain intercepts equal over time, estimate factor mean at T2 and T3

ScalarCSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts NOW CONSTRAINED EQUAL OVER TIME VIA SAME LABELS EXCEPT v1T1 v5T1 v4T1
v1T1 ~ I1T1*1; v2T1 ~ I2*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;  v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;  v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances all freely estimated at each occasion with labels
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3T1*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1T2*v1T2; v2T2 ~~ E2T2*v2T2;  v3T2 ~~ E3T2*v3T2; v4T2 ~~ E4T2*v4T2; v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6T2*v6T2
v1T3 ~~ E1T3*v1T3; v2T3 ~~ E2T3*v2T3;  v3T3 ~~ E3T3*v3T3; v4T3 ~~ E4T3*v4T3; v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6T3*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 only, still estimated at T2 and T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 ONLY, NOW ESTIMATED AT T2 AND T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
ScalarC=lavaan(model=ScalarCSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(ScalarC, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ScalarC, op="~1", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarC better than ScalarB?")
anova(ScalarC, ScalarB)

print("LRT: Is ScalarC worse than MetricB?")
anova(MetricB, ScalarC)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 3d: Scalar Invariance Model (no v1T1 v5T1 v4T1 v2T1)
# constrain intercepts equal over time, estimate factor mean at T2 and T3

ScalarDSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts NOW CONSTRAINED EQUAL OVER TIME VIA SAME LABELS EXCEPT v1T1 v5T1 v4T1 v2T1
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;    v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;    v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances all freely estimated at each occasion with labels
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3T1*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1T2*v1T2; v2T2 ~~ E2T2*v2T2;  v3T2 ~~ E3T2*v3T2; v4T2 ~~ E4T2*v4T2; v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6T2*v6T2
v1T3 ~~ E1T3*v1T3; v2T3 ~~ E2T3*v2T3;  v3T3 ~~ E3T3*v3T3; v4T3 ~~ E4T3*v4T3; v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6T3*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 only, still estimated at T2 and T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 ONLY, NOW ESTIMATED AT T2 AND T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
ScalarD=lavaan(model=ScalarDSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(ScalarD, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ScalarD, op="~1", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarD better than ScalarC?")
anova(ScalarD, ScalarC)

print("LRT: Is ScalarD worse than MetricB?")
anova(MetricB, ScalarD)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 4a: Residual Variance Invariance Model
# constrain residual variances equal over time (where possible)

ResVarASyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts still constrained equal via same labels except v1T1 v5T1 v4T1 v2T1
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;    v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;    v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances NOW CONSTRAINED EQUAL VIA SAME LABELS WHERE POSSIBLE (no v1T1 v5T1 v4T1 v2T1)
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6*v6T1
v1T2 ~~ E1*v1T2;   v2T2 ~~ E2*v2T2;    v3T2 ~~ E3*v3T2; v4T2 ~~ E4*v4T2;   v5T2 ~~ E5*v5T2;   v6T2 ~~ E6*v6T2
v1T3 ~~ E1*v1T3;   v2T3 ~~ E2*v2T3;    v3T3 ~~ E3*v3T3; v4T3 ~~ E4*v4T3;   v5T3 ~~ E5*v5T3;   v6T3 ~~ E6*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 only, still estimated at T2 and T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 only, still estimated at T2 and T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
ResVarA=lavaan(model=ResVarASyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(ResVarA, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ResVarA, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ResVarA worse than ScalarD?")
anova(ScalarD, ResVarA)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 4b: Residual Variance Invariance Model
# constrain residual variances equal over time (where possible)

ResVarBSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts still constrained equal via same labels except v1T1 v5T1 v4T1 v2T1
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;    v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;    v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances NOW CONSTRAINED EQUAL VIA SAME LABELS WHERE POSSIBLE (no v1T1 v5T1 v4T1 v2T1, v5T2-v5T3)
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6*v6T1
v1T2 ~~ E1*v1T2;   v2T2 ~~ E2*v2T2;    v3T2 ~~ E3*v3T2; v4T2 ~~ E4*v4T2;   v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6*v6T2
v1T3 ~~ E1*v1T3;   v2T3 ~~ E2*v2T3;    v3T3 ~~ E3*v3T3; v4T3 ~~ E4*v4T3;   v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 only, still estimated at T2 and T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 only, still estimated at T2 and T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
ResVarB=lavaan(model=ResVarBSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(ResVarB, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ResVarB, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ResVarB better than ResVarA?")
anova(ResVarB, ResVarA)

print("LRT: Is ResVarB worse than ScalarD?")
anova(ScalarD, ResVarB)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Measurement Invariance Model 4c: Residual Variance Invariance Model
# constrain residual variances equal over time (where possible)

ResVarCSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts still constrained equal via same labels except v1T1 v5T1 v4T1 v2T1
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;    v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;    v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances NOW CONSTRAINED EQUAL VIA SAME LABELS WHERE POSSIBLE (no v1T1 v5T1 v4T1 v2T1, v5T2-v5T3, v6T2-v6T3)
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1*v1T2;   v2T2 ~~ E2*v2T2;    v3T2 ~~ E3*v3T2; v4T2 ~~ E4*v4T2;   v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6*v6T2
v1T3 ~~ E1*v1T3;   v2T3 ~~ E2*v2T3;    v3T3 ~~ E3*v3T3; v4T3 ~~ E4*v4T3;   v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 only, still estimated at T2 and T3
Time1 ~~ 1*Time1; Time2 ~~ Time2; Time3 ~~ Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 only, still estimated at T2 and T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
ResVarC=lavaan(model=ResVarCSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(ResVarC, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ResVarC, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ResVarC better than ResVarB?")
anova(ResVarC, ResVarB)

print("LRT: Is ResVarC worse than ScalarD?")
anova(ScalarD, ResVarC)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Structural Invariance Model 5a: Factor Variance Invariance Model
# constrain factor variances equal over time (leave all other measurement partial constraints)

FactorVarSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts still constrained equal via same labels except v1T1 v5T1 v4T1 v2T1
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;    v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;    v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances still constrained equal via same labels (no v1T1 v5T1 v4T1 v2T1, v5T2-v5T3, v6T2-v6T3)
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1*v1T2;   v2T2 ~~ E2*v2T2;    v3T2 ~~ E3*v3T2; v4T2 ~~ E4*v4T2;   v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6*v6T2
v1T3 ~~ E1*v1T3;   v2T3 ~~ E2*v2T3;    v3T3 ~~ E3*v3T3; v4T3 ~~ E4*v4T3;   v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 AND AT T2 and T3 TO TEST INVARIANCE
Time1 ~~ 1*Time1; Time2 ~~ 1*Time2; Time3 ~~ 1*Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 only, still estimated at T2 and T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances all freely estimated
Time1 ~~ Time2 + Time3; Time2 ~~ Time3 
#================================================================================================================
"
FactorVar=lavaan(model=FactorVarSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(FactorVar, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=FactorVar, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is FactorVar worse than ResVarC?")
anova (ResVarC, FactorVar)

txtComment(" ") # insert blank space in output

#################################################################################################################
# Structural Invariance Model 6a: Factor Covariance Invariance Model
# constrain factor covariances equal over time (leave all other measurement partial constraints)

FactorCoVarSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts still constrained equal via same labels except v1T1 v5T1 v4T1 v2T1
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;    v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;    v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances still constrained equal via same labels (no v1T1 v5T1 v4T1 v2T1, v5T2-v5T3, v6T2-v6T3)
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1*v1T2;   v2T2 ~~ E2*v2T2;    v3T2 ~~ E3*v3T2; v4T2 ~~ E4*v4T2;   v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6*v6T2
v1T3 ~~ E1*v1T3;   v2T3 ~~ E2*v2T3;    v3T3 ~~ E3*v3T3; v4T3 ~~ E4*v4T3;   v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 AND AT T2 and T3 TO ENFORCE INVARIANCE
Time1 ~~ 1*Time1; Time2 ~~ 1*Time2; Time3 ~~ 1*Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 only, still estimated at T2 and T3
Time1 ~ 0; Time2 ~ 1; Time3 ~ 1
#================================================================================================================
# Factor covariances NOW CONSTRAINED EQUAL OVER TIME VIA SAME LABEL
Time1 ~~ Fcov*Time2 + Fcov*Time3; Time2 ~~ Fcov*Time3 
#================================================================================================================
"
FactorCoVar=lavaan(model=FactorCoVarSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(FactorCoVar, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=FactorCoVar, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is FactorCoVar worse than FactorVar?")
anova (FactorVar, FactorCoVar)

txtComment(" ") # insert blank space in output


#################################################################################################################
# Structural Invariance Model 7a: Factor Mean Invariance Model
# constrain factor means equal over time (leave all other measurement partial constraints)

FactorMeanSyntax="
#================================================================================================================
# Factor loadings still constrained equal via same labels except v1T1
Time1 =~ L1T1*v1T1 + L2*v2T1 + L3*v3T1 + L4*v4T1 + L5*v5T1 + L6*v6T1
Time2 =~ L1*v1T2   + L2*v2T2 + L3*v3T2 + L4*v4T2 + L5*v5T2 + L6*v6T2
Time3 =~ L1*v1T3   + L2*v2T3 + L3*v3T3 + L4*v4T3 + L5*v5T3 + L6*v6T3
#================================================================================================================
# Item intercepts still constrained equal via same labels except v1T1 v5T1 v4T1 v2T1
v1T1 ~ I1T1*1; v2T1 ~ I2T1*1;  v3T1 ~ I3*1; v4T1 ~ I4T1*1; v5T1 ~ I5T1*1; v6T1 ~ I6*1 
v1T2 ~ I1*1;   v2T2 ~ I2*1;    v3T2 ~ I3*1; v4T2 ~ I4*1;   v5T2 ~ I5*1;   v6T2 ~ I6*1 
v1T3 ~ I1*1;   v2T3 ~ I2*1;    v3T3 ~ I3*1; v4T3 ~ I4*1;   v5T3 ~ I5*1;   v6T3 ~ I6*1 
#================================================================================================================
# Residual variances still constrained equal via same labels (no v1T1 v5T1 v4T1 v2T1, v5T2-v5T3, v6T2-v6T3)
v1T1 ~~ E1T1*v1T1; v2T1 ~~ E2T1*v2T1;  v3T1 ~~ E3*v3T1; v4T1 ~~ E4T1*v4T1; v5T1 ~~ E5T1*v5T1; v6T1 ~~ E6T1*v6T1
v1T2 ~~ E1*v1T2;   v2T2 ~~ E2*v2T2;    v3T2 ~~ E3*v3T2; v4T2 ~~ E4*v4T2;   v5T2 ~~ E5T2*v5T2; v6T2 ~~ E6*v6T2
v1T3 ~~ E1*v1T3;   v2T3 ~~ E2*v2T3;    v3T3 ~~ E3*v3T3; v4T3 ~~ E4*v4T3;   v5T3 ~~ E5T3*v5T3; v6T3 ~~ E6*v6T3
#================================================================================================================
# Residual covariances for same indicator over time freely estimated at each occasion with labels
v1T1 ~~ C1T12*v1T2 + C1T13*v1T3; v1T2 ~~ C1T23*v1T3; 
v2T1 ~~ C2T12*v2T2 + C2T13*v2T3; v2T2 ~~ C2T23*v2T3; 
v3T1 ~~ C3T12*v3T2 + C3T13*v3T3; v3T2 ~~ C3T23*v3T3; 
v4T1 ~~ C4T12*v4T2 + C4T13*v4T3; v4T2 ~~ C4T23*v4T3; 
v5T1 ~~ C5T12*v5T2 + C5T13*v5T3; v5T2 ~~ C5T23*v5T3; 
v6T1 ~~ C6T12*v6T2 + C6T13*v6T3; v6T2 ~~ C6T23*v6T3; 
#================================================================================================================
# Factor variance fixed to 1 for identification at T1 AND AT T2 and T3 TO ENFORCE INVARIANCE
Time1 ~~ 1*Time1; Time2 ~~ 1*Time2; Time3 ~~ 1*Time3
#================================================================================================================
# Factor mean fixed to zero for identification at T1 AND EQUAL AT T2 and T3 TO TEST INVARIANCE
Time1 ~ 0; Time2 ~ Fmean*1; Time3 ~ Fmean*1
#================================================================================================================
# Factor covariances still constrained equal over time via same label
Time1 ~~ Fcov*Time2 + Fcov*Time3; Time2 ~~ Fcov*Time3  
#================================================================================================================
"
FactorMean=lavaan(model=FactorMeanSyntax, data=Example7b, estimator="MLR", mimic="mplus")
summary(FactorMean, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=FactorMean, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is FactorMean worse than FactorCoVar?")
anova (FactorCoVar, FactorMean)

txtComment(" ") # insert blank space in output


#################################################################################################################

# Close external file of saved results
txtStop()


